@@ -14,6 +14,7 @@
 #    under the License.
 import collections
 import logging
+import os
 
 import pkg_resources
 import requests
@@ -231,7 +232,7 @@ class Sushy(base.ResourceBase):
         return system.SystemCollection(
             self._conn, self._systems_path,
             redfish_version=self.redfish_version,
-            registries=self.lazy_registries)
+            registries=self.lazy_registries, root=self)
 
     def get_system(self, identity=None):
         """Given the identity return a System object
@@ -254,7 +255,7 @@ class Sushy(base.ResourceBase):
 
         return system.System(self._conn, identity,
                              redfish_version=self.redfish_version,
-                             registries=self.lazy_registries)
+                             registries=self.lazy_registries, root=self)
 
     def get_chassis_collection(self):
         """Get the ChassisCollection object
@@ -269,7 +270,8 @@ class Sushy(base.ResourceBase):
 
         return chassis.ChassisCollection(self._conn, self._chassis_path,
                                          redfish_version=self.redfish_version,
-                                         registries=self.lazy_registries)
+                                         registries=self.lazy_registries,
+                                         root=self)
 
     def get_chassis(self, identity=None):
         """Given the identity return a Chassis object
@@ -292,7 +294,7 @@ class Sushy(base.ResourceBase):
 
         return chassis.Chassis(self._conn, identity,
                                redfish_version=self.redfish_version,
-                               registries=self.lazy_registries)
+                               registries=self.lazy_registries, root=self)
 
     def get_fabric_collection(self):
         """Get the FabricCollection object
@@ -307,7 +309,8 @@ class Sushy(base.ResourceBase):
 
         return fabric.FabricCollection(self._conn, self._fabrics_path,
                                        redfish_version=self.redfish_version,
-                                       registries=self.lazy_registries)
+                                       registries=self.lazy_registries,
+                                       root=self)
 
     def get_fabric(self, identity):
         """Given the identity return a Fabric object
@@ -317,7 +320,7 @@ class Sushy(base.ResourceBase):
         """
         return fabric.Fabric(self._conn, identity,
                              redfish_version=self.redfish_version,
-                             registries=self.lazy_registries)
+                             registries=self.lazy_registries, root=self)
 
     def get_manager_collection(self):
         """Get the ManagerCollection object
@@ -332,7 +335,8 @@ class Sushy(base.ResourceBase):
 
         return manager.ManagerCollection(self._conn, self._managers_path,
                                          redfish_version=self.redfish_version,
-                                         registries=self.lazy_registries)
+                                         registries=self.lazy_registries,
+                                         root=self)
 
     def get_manager(self, identity=None):
         """Given the identity return a Manager object
@@ -354,7 +358,7 @@ class Sushy(base.ResourceBase):
 
         return manager.Manager(self._conn, identity,
                                redfish_version=self.redfish_version,
-                               registries=self.lazy_registries)
+                               registries=self.lazy_registries, root=self)
 
     def get_session_service(self):
         """Get the SessionService object
@@ -369,14 +373,19 @@ class Sushy(base.ResourceBase):
 
         return sessionservice.SessionService(
             self._conn, self._session_service_path,
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version, root=self)
 
     def get_sessions_path(self):
         """Returns the Sessions url"""
-
+        # NOTE(TheJulia): This method is the standard discovery method
+        # advised by the DMTF DSP0266 standard to find the session service.
         try:
             links_url = self.json.get('Links')
-            return links_url['Sessions']['@odata.id']
+            sessions_uri = links_url['Sessions']['@odata.id']
+            # Save the session URL for post detection and prevention
+            # of recursive autentication attempts.
+            self._conn._sessions_uri = sessions_uri
+            return sessions_uri
         except (TypeError, KeyError):
             raise exceptions.MissingAttributeError(
                 attribute='Links/Sessions/@data.id', resource=self.path)
@@ -390,7 +399,64 @@ class Sushy(base.ResourceBase):
         return session.Session(
             self._conn, identity,
             redfish_version=self.redfish_version,
-            registries=self.lazy_registries)
+            registries=self.lazy_registries, root=self)
+
+    def create_session(self, username=None, password=None):
+        """Creates a session without invoking SessionService.
+
+        For use when a new connection is to be established. Removes prior
+        Session and authentication data before making the request.
+
+        :param username: The username to utilize to create a session with the
+                         remote endpoint.
+        :param password: The password to utilize to create a session with the
+                         remote endpoint.
+        :returns: A session key and uri in the form of a tuple
+        :raises: MissingXAuthToken
+        :raises: ConnectionError
+        :raises: AccessError
+        :raises: HTTPError
+        :raises: MissingAttributeError
+        """
+
+        # Explicitly removes in-client session data to proceed. This prevents
+        # AccessErrors as prior authentication shouldn't be submitted with a
+        # new authentication attempt, and doing so with old/invalid session
+        # data can result in an AccessError being raised.
+        self._conn._session.auth = None
+        if 'X-Auth-Token' in self._conn._session.headers:
+            # Delete the token value that was saved to the session
+            # as otherwise we would end up with a dictionary containing
+            # a {'X-Auth-Token': null} being sent across to the remote
+            # bmc.
+            del self._conn._session.headers['X-Auth-Token']
+        try:
+            session_service_path = self.get_sessions_path()
+        except (exceptions.MissingAttributeError, exceptions.AccessError):
+            # Guesses path based upon DMTF standard, in the event
+            # the links resource on the service root is incorrect.
+            session_service_path = os.path.join(self._path,
+                                                'SessionService/Sessions')
+            LOG.warning('Could not discover the Session service path, '
+                        'falling back to %s.',
+                        session_service_path)
+            session_url = self._root_prefix + 'SessionService/Sessions'
+            self._conn._sessions_uri = session_url
+
+        data = {'UserName': username, 'Password': password}
+        LOG.debug("Requesting new session from %s.",
+                  session_service_path)
+        rsp = self._conn.post(session_service_path, data=data)
+        session_key = rsp.headers.get('X-Auth-Token')
+        if session_key is None:
+            raise exceptions.MissingXAuthToken(
+                method='POST', url=session_service_path, response=rsp)
+
+        session_uri = rsp.headers.get('Location')
+        if session_uri is None:
+            LOG.warning("Received X-Auth-Token but NO session uri.")
+
+        return session_key, session_uri
 
     def get_update_service(self):
         """Get the UpdateService object
@@ -404,7 +470,7 @@ class Sushy(base.ResourceBase):
         return updateservice.UpdateService(
             self._conn, self._update_service_path,
             redfish_version=self.redfish_version,
-            registries=self.lazy_registries)
+            registries=self.lazy_registries, root=self)
 
     def get_task_service(self):
         """Get the TaskService object
@@ -414,7 +480,7 @@ class Sushy(base.ResourceBase):
         return taskservice.TaskService(
             self._conn, utils.get_sub_resource_path_by(self, 'Tasks'),
             redfish_version=self.redfish_version,
-            registries=self.lazy_registries)
+            registries=self.lazy_registries, root=self)
 
     def _get_registry_collection(self):
         """Get MessageRegistryFileCollection object
@@ -429,7 +495,7 @@ class Sushy(base.ResourceBase):
             return message_registry_file.MessageRegistryFileCollection(
                 self._conn,
                 self._registries_path,
-                redfish_version=self.redfish_version)
+                redfish_version=self.redfish_version, root=self)
 
     def get_composition_service(self):
         """Get the CompositionService object
@@ -446,7 +512,7 @@ class Sushy(base.ResourceBase):
         return compositionservice.CompositionService(
             self._conn, self._composition_service_path,
             redfish_version=self.redfish_version,
-            registries=self.lazy_registries)
+            registries=self.lazy_registries, root=self)
 
     def get_event_service(self):
         """Get the EventService object
@@ -461,7 +527,7 @@ class Sushy(base.ResourceBase):
         return eventservice.EventService(
             self._conn, self._event_service_path,
             redfish_version=self.redfish_version,
-            registries=self.lazy_registries)
+            registries=self.lazy_registries, root=self)
 
     def _get_standard_message_registry_collection(self):
         """Load packaged standard message registries
