@@ -16,6 +16,7 @@ from http import client as http_client
 import json
 from unittest import mock
 
+import requests
 
 import sushy
 from sushy import exceptions
@@ -104,6 +105,20 @@ class VirtualMediaTestCase(base.TestCase):
         )
         self.assertTrue(self.sys_virtual_media._is_stale)
 
+    def test_insert_media_credentials(self):
+        self.assertFalse(self.sys_virtual_media._is_stale)
+        self.sys_virtual_media.insert_media(
+            "https://www.dmtf.org/freeImages/Sardine.img", True, False,
+            username="admin", password="pwd")
+        self.sys_virtual_media._conn.post.assert_called_once_with(
+            ("/redfish/v1/Managers/BMC/VirtualMedia/Floppy1/Actions"
+             "/VirtualMedia.InsertMedia"),
+            data={"Image": "https://www.dmtf.org/freeImages/Sardine.img",
+                  "WriteProtected": False,
+                  "UserName": "admin", "Password": "pwd"}
+        )
+        self.assertTrue(self.sys_virtual_media._is_stale)
+
     def test_insert_media_fallback(self):
         self.conn.get.return_value.headers = {'Allow': 'GET,HEAD,PATCH'}
         self.sys_virtual_media._actions.insert_media = None
@@ -142,6 +157,20 @@ class VirtualMediaTestCase(base.TestCase):
             headers={"If-Match": '"3d7b8a7360bf2941d"'})
         self.assertTrue(self.sys_virtual_media._is_stale)
 
+    @mock.patch.object(requests, 'post', autospec=True)
+    def test_is_transfer_protocol_required(self, mock_post):
+        with open('sushy/tests/unit/json_samples/'
+                  'transfer_proto_required_error.json') as f:
+            response_obj = json.load(f)
+        response = mock.MagicMock()
+        response.status_code = 400
+        response.code = "Base.1.5.ActionParameterMissing"
+        response.body = response_obj['error']
+        response.raise_for_status.side_effect = requests.exceptions.HTTPError
+        mock_post.return_value = response
+        retval = self.sys_virtual_media.is_transfer_protocol_required(response)
+        self.assertTrue(retval)
+
     def test_eject_media_none(self):
         self.sys_virtual_media._actions.eject_media = None
         self.assertRaisesRegex(
