@@ -784,13 +784,15 @@ class SystemCollectionTestCase(base.TestCase):
         self.sys_col.get_member('/redfish/v1/Systems/437XR1138R2')
         mock_system.assert_called_once_with(
             self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
-            self.sys_col.redfish_version, None)
+            redfish_version=self.sys_col.redfish_version, registries=None,
+            root=self.sys_col.root)
 
     @mock.patch.object(system, 'System', autospec=True)
     def test_get_members(self, mock_system):
         members = self.sys_col.get_members()
         mock_system.assert_called_once_with(
             self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
-            self.sys_col.redfish_version, None)
+            redfish_version=self.sys_col.redfish_version, registries=None,
+            root=self.sys_col.root)
         self.assertIsInstance(members, list)
         self.assertEqual(1, len(members))
